@@ -1,5 +1,6 @@
 # We need to do things in all phases without interference, so avoid
 # using Test::More and just implement our own simple test routines.
+package t::test;
 use strict;
 $|++;
 {
@@ -12,17 +13,27 @@ $|++;
 my $had_error;
 my $test_num;
 my $done;
+BEGIN {
+    if ($INC{'threads.pm'}) {
+        require threads::shared;
+        threads::shared::share(\$had_error);
+        threads::shared::share(\$test_num);
+        threads::shared::share(\$done);
+    }
+}
 END { $? = $done ? 0 : 1 }
 sub ::ok ($;$) {
-  $had_error++, print "not " if !$_[0];
+  print "not " if !$_[0];
   print "ok " . ++$test_num;
   print " - $_[1]" if defined $_[1];
+  print " # TODO $::TODO" if defined $::TODO;
+  $had_error++ if !$_[0] && !$::TODO;
   print "\n";
   !!$_[0]
 }
 sub ::is ($$;$) {
   my $out = ::ok $_[0] eq $_[1], $_[2]
-    or warn "# $_[0] ne $_[1]\n";
+    or print "# $_[0] ne $_[1]\n";
   $out;
 }
 sub ::skip ($;$) {
